@@ -112,11 +112,12 @@ FontFileMakeDir(const char *dirName, int size)
     const char		*attrib;
     int			attriblen;
 
-#if !defined(WIN32)
     attrib = strchr(dirName, ':');
-#else
-    /* OS/2 uses the colon in the drive letter descriptor, skip this */
-    attrib = strchr(dirName+2, ':');
+#if defined(WIN32)
+    if (attrib && attrib - dirName == 1) {
+	/* WIN32 uses the colon in the drive letter descriptor, skip this */
+	attrib = strchr(dirName + 2, ':');
+    }
 #endif
     if (attrib) {
 	dirlen = attrib - dirName;
@@ -125,10 +126,7 @@ FontFileMakeDir(const char *dirName, int size)
 	dirlen = strlen(dirName);
 	attriblen = 0;
     }
-    if (dirName[dirlen - 1] != '/')
-#ifdef NCD
-    if (dirlen)     /* leave out slash for builtins */
-#endif
+    if (dirlen && dirName[dirlen - 1] != '/')
 	needslash = 1;
     dir = malloc(sizeof *dir + dirlen + needslash + 1 +
 		 (attriblen ? attriblen + 1 : 0));
